<!doctype html>
<html>
<head>
  <title>{% block title %}{% endblock %}</title>
  <meta charset="utf-8"/>
  {% block css %}{% endblock %}
</head>
<body>
  {% block body %}{% endblock %}

  {% block js %}{% endblock %}
</body>
</html>
